from enum import StrEnum

from pydantic import GetJsonSchemaHandler
from pydantic.json_schema import JsonSchemaValue
from pydantic_core import CoreSchema


class Resource(StrEnum):
    PROFILE = "profile"
    ACTIVITY = "activity"
    SLEEP = "sleep"
    BODY = "body"
    WORKOUTS = "workouts"
    WORKOUT_STREAM = "workout_stream"
    CONNECTION = "connection"
    ORDER = "order"
    RESULT = "result"
    APPOINTMENT = "appointment"
    GLUCOSE = "glucose"
    HEARTRATE = "heartrate"
    HEARTRATE_VARIABILITY = "hrv"
    HRV = "hrv"
    IGE = "ige"
    IGG = "igg"
    BLOOD_OXYGEN = "blood_oxygen"
    BLOOD_PRESSURE = "blood_pressure"
    CHOLESTEROL = "cholesterol"
    DEVICE = "device"
    DEVICE_LEGACY = "device_legacy"
    WEIGHT = "weight"
    FAT = "fat"
    BODY_TEMPERATURE = "body_temperature"
    BODY_TEMPERATURE_DELTA = "body_temperature_delta"
    MEALS = "meal"
    WATER = "water"
    CAFFEINE = "caffeine"
    MINDFULNESS_MINUTES = "mindfulness_minutes"
    STEPS = "steps"
    CALORIES_ACTIVE = "calories_active"
    DISTANCE = "distance"
    FLOORS_CLIMBED = "floors_climbed"
    RESPIRATORY_RATE = "respiratory_rate"
    VO2_MAX = "vo2_max"
    CALORIES_BASAL = "calories_basal"
    STRESS_LEVEL = "stress_level"
    MENSTRUAL_CYCLE = "menstrual_cycle"
    SLEEP_CYCLE = "sleep_cycle"

    ELECTROCARDIOGRAM = "electrocardiogram"
    ELECTROCARDIOGRAM_VOLTAGE = "electrocardiogram_voltage"
    AFIB_BURDEN = "afib_burden"
    HEART_RATE_ALERT = "heart_rate_alert"

    STAND_HOUR = "stand_hour"
    STAND_DURATION = "stand_duration"
    SLEEP_APNEA_ALERT = "sleep_apnea_alert"
    SLEEP_BREATHING_DISTURBANCE = "sleep_breathing_disturbance"
    WHEELCHAIR_PUSH = "wheelchair_push"
    FORCED_EXPIRATORY_VOLUME_1 = "forced_expiratory_volume_1"
    FORCED_VITAL_CAPACITY = "forced_vital_capacity"
    PEAK_EXPIRATORY_FLOW_RATE = "peak_expiratory_flow_rate"
    INHALER_USAGE = "inhaler_usage"
    FALL = "fall"
    UV_EXPOSURE = "uv_exposure"
    DAYLIGHT_EXPOSURE = "daylight_exposure"
    HANDWASHING = "handwashing"
    BASAL_BODY_TEMPERATURE = "basal_body_temperature"
    HEART_RATE_RECOVERY_ONE_MINUTE = "heart_rate_recovery_one_minute"
    BODY_MASS_INDEX = "body_mass_index"
    LEAN_BODY_MASS = "lean_body_mass"
    WAIST_CIRCUMFERENCE = "waist_circumference"

    WORKOUT_DISTANCE = "workout_distance"
    WORKOUT_SWIMMING_STROKE = "workout_swimming_stroke"

    WORKOUT_DURATION = "workout_duration"
    INSULIN_INJECTION = "insulin_injection"
    CARBOHYDRATES = "carbohydrates"
    NOTE = "note"

    # Deprecated resource types:
    SLEEP_STREAM = "sleep_stream"
    HYPNOGRAM = "hypnogram"

    @classmethod
    def __get_pydantic_json_schema__(
        cls, core_schema: CoreSchema, handler: GetJsonSchemaHandler
    ) -> JsonSchemaValue:
        schema = handler(core_schema)
        # SDK Codegen backward compatibility
        schema["title"] = "ClientFacingResource"
        schema["x-fern-type-name"] = "ClientFacingResource"
        return schema

    @property
    def is_timeseries(self) -> bool:
        match self:
            case (
                Resource.PROFILE
                | Resource.ACTIVITY
                | Resource.SLEEP
                | Resource.WORKOUTS
                | Resource.WORKOUT_STREAM
                | Resource.CONNECTION
                | Resource.ORDER
                | Resource.DEVICE
                | Resource.DEVICE_LEGACY
                | Resource.MEALS
                | Resource.BODY
                | Resource.APPOINTMENT
                | Resource.RESULT
                | Resource.MENSTRUAL_CYCLE
                | Resource.SLEEP_STREAM
                | Resource.SLEEP_CYCLE
                | Resource.ELECTROCARDIOGRAM
            ):
                return False
            case _:
                return True



from enum import StrEnum


class Providers(StrEnum):
    OURA = "oura"
    FITBIT = "fitbit"
    GARMIN = "garmin"
    WHOOP = "whoop"
    STRAVA = "strava"
    RENPHO = "renpho"
    PELOTON = "peloton"
    WAHOO = "wahoo"
    ZWIFT = "zwift"
    FREESTYLE_LIBRE = "freestyle_libre"
    ABBOTT_LIBREVIEW = "abbott_libreview"
    TANDEM_SOURCE = "tandem_source"
    FREESTYLE_LIBRE_BLE = "freestyle_libre_ble"
    EIGHT_SLEEP = "eight_sleep"
    WITHINGS = "withings"
    APPLE_HEALTH_KIT = "apple_health_kit"
    MANUAL = "manual"
    IHEALTH = "ihealth"
    GOOGLE_FIT = "google_fit"
    BUERER_API = "beurer_api"
    BEURER_BLE = "beurer_ble"
    OMRON = "omron"
    OMRON_BLE = "omron_ble"
    ONETOUCH_BLE = "onetouch_ble"
    ACCUCHEK_BLE = "accuchek_ble"
    CONTOUR_BLE = "contour_ble"
    DEXCOM = "dexcom"
    DEXCOM_V3 = "dexcom_v3"
    HAMMERHEAD = "hammerhead"
    MY_FITNESS_PAL = "my_fitness_pal"
    HEALTH_CONNECT = "health_connect"
    SAMSUNG_HEALTH = "samsung_health"
    POLAR = "polar"
    CRONOMETER = "cronometer"
    KARDIA = "kardia"
    WHOOP_V2 = "whoop_v2"
    ULTRAHUMAN = "ultrahuman"
    MY_FITNESS_PAL_V2 = "my_fitness_pal_v2"
    MAP_MY_FITNESS = "map_my_fitness"
    RUNKEEPER = "runkeeper"


class Labs(StrEnum):
    AYUMETRIX = "ayumetrix"
    SPIRIPLEX = "spiriplex"
    USSL = "ussl"
    QUEST = "quest"
    SONORA_QUEST = "sonora_quest"
    LABCORP = "labcorp"
    BIOREFERENCE = "bioreference"
    BIOTEK = "us_biotek"
    MANUAL = "manual"
    SANOCARDIO = "sanocardio"
    IHD = "ihd"
    NEXUS = "nexus"
    MYUTI = "my_uti"
    CRL = "crl"



import os
from typing import Literal

VitalEnvironmentT = Literal["sandbox", "production"]
VitalRegionT = Literal["us", "eu"]


def api_base_url(environment: VitalEnvironmentT, region: VitalRegionT) -> str:
    if base_url := os.getenv("VITALX_BASE_URL"):
        return base_url

    match region:
        case "us":
            match environment:
                case "production":
                    return "https://api.tryvital.io/"
                case "sandbox":
                    return "https://api.sandbox.tryvital.io/"
        case "eu":
            match environment:
                case "production":
                    return "https://api.eu.tryvital.io/"
                case "sandbox":
                    return "https://api.sandbox.eu.tryvital.io/"



from enum import StrEnum


class USState(StrEnum):
    AL = "AL"
    AK = "AK"
    AZ = "AZ"
    AR = "AR"
    CA = "CA"
    CO = "CO"
    CT = "CT"
    DE = "DE"
    FL = "FL"
    GA = "GA"
    HI = "HI"
    ID = "ID"
    IL = "IL"
    IN = "IN"
    IA = "IA"
    KS = "KS"
    KY = "KY"
    LA = "LA"
    ME = "ME"
    MD = "MD"
    MA = "MA"
    MI = "MI"
    MN = "MN"
    MS = "MS"
    MO = "MO"
    MT = "MT"
    NE = "NE"
    NV = "NV"
    NH = "NH"
    NJ = "NJ"
    NM = "NM"
    NY = "NY"
    NC = "NC"
    ND = "ND"
    OH = "OH"
    OK = "OK"
    OR = "OR"
    PA = "PA"
    RI = "RI"
    SC = "SC"
    SD = "SD"
    TN = "TN"
    TX = "TX"
    UT = "UT"
    VT = "VT"
    VA = "VA"
    WA = "WA"
    WV = "WV"
    WI = "WI"
    WY = "WY"


class USTerritory(StrEnum):
    DC = "DC"
    AS = "AS"
    GU = "GU"
    MP = "MP"
    PR = "PR"
    UM = "UM"
    VI = "VI"


USStateOrTerritory = USState | USTerritory


STATE_NAME_TO_CODE: dict[str, str] = {
    "alabama": "AL",
    "alaska": "AK",
    "arizona": "AZ",
    "arkansas": "AR",
    "california": "CA",
    "colorado": "CO",
    "connecticut": "CT",
    "delaware": "DE",
    "florida": "FL",
    "georgia": "GA",
    "hawaii": "HI",
    "idaho": "ID",
    "illinois": "IL",
    "indiana": "IN",
    "iowa": "IA",
    "kansas": "KS",
    "kentucky": "KY",
    "louisiana": "LA",
    "maine": "ME",
    "maryland": "MD",
    "massachusetts": "MA",
    "michigan": "MI",
    "minnesota": "MN",
    "mississippi": "MS",
    "missouri": "MO",
    "montana": "MT",
    "nebraska": "NE",
    "nevada": "NV",
    "new hampshire": "NH",
    "new jersey": "NJ",
    "new mexico": "NM",
    "new york": "NY",
    "north carolina": "NC",
    "north dakota": "ND",
    "ohio": "OH",
    "oklahoma": "OK",
    "oregon": "OR",
    "pennsylvania": "PA",
    "rhode island": "RI",
    "south carolina": "SC",
    "south dakota": "SD",
    "tennessee": "TN",
    "texas": "TX",
    "utah": "UT",
    "vermont": "VT",
    "virginia": "VA",
    "washington": "WA",
    "west virginia": "WV",
    "wisconsin": "WI",
    "wyoming": "WY",
    "district of columbia": "DC",
    "american samoa": "AS",
    "guam": "GU",
    "northern mariana islands": "MP",
    "puerto rico": "PR",
    "united states minor outlying islands": "UM",
    "u.s. virgin islands": "VI",
}


def parse_us_state_or_territory(value: str | None) -> USStateOrTerritory | None:
    """
    Parse a US state or territory from either a full name or 2-letter code. Case insensitive.

    Args:
        value: A state/territory name (e.g., "New Mexico", "Puerto Rico") or code (e.g., "NM", "PR").

    Returns:
        The corresponding USState or USTerritory enum value, or None if not found.
    """
    if not value:
        return None

    normalized = value.strip().lower()
    if len(normalized) == 2:
        code = normalized.upper()
        try:
            return USState(code)
        except ValueError:
            pass
        try:
            return USTerritory(code)
        except ValueError:
            return None

    code = STATE_NAME_TO_CODE.get(normalized)
    if code:
        try:
            return USState(code)
        except ValueError:
            return USTerritory(code)

    return None


def parse_us_state(value: str | None) -> USState | None:
    """
    Parse a US state (not territory) from either a full name or 2-letter code. Case insensitive.

    Args:
        value: A state name (e.g., "New Mexico") or code (e.g., "NM").

    Returns:
        The corresponding USState enum value, or None if not found.
    """
    result = parse_us_state_or_territory(value)
    if isinstance(result, USState):
        return result
    return None

